@@ -1,6 +1,7 @@
 package Chrome::DevToolsProtocol::Transport::AnyEvent;
 use strict;
 use Filter::signatures;
+use Moo 2;
 no warnings 'experimental::signatures';
 use feature 'signatures';
 use Scalar::Util 'weaken';
@@ -11,7 +12,7 @@ use AnyEvent;
 use AnyEvent::WebSocket::Client;
 use AnyEvent::Future qw(as_future_cb);
 
-our $VERSION = '0.36';
+our $VERSION = '0.37';
 our @CARP_NOT = ();
 
 =head1 NAME
@@ -29,15 +30,18 @@ Chrome::DevToolsProtocol::Transport::AnyEvent - AnyEvent backend for Chrome comm
 
 =cut
 
-sub new( $class, %options ) {
-    my $self = \%options;
-    bless $self => $class;
-    $self
-}
+has 'type' => (
+    is => 'ro',
+    default => 'websocket'
+);
 
-sub connection( $self ) {
-    $self->{connection}
-}
+has 'connection' => (
+    is => 'rw',
+);
+
+has 'ws_client' => (
+    is => 'rw',
+);
 
 sub connect( $self, $handler, $got_endpoint, $logger ) {
     weaken $handler;
@@ -52,10 +56,10 @@ sub connect( $self, $handler, $got_endpoint, $logger ) {
 
         my $res = $self->future;
         $logger->('debug',"Connecting to $endpoint");
-        $self->{ws_client} = AnyEvent::WebSocket::Client->new(
+        $self->ws_client( AnyEvent::WebSocket::Client->new(
             max_payload_size => 0, # allow unlimited size for messages
-        );
-        $self->{ws_client}->connect( $endpoint )->cb( sub {
+        ));
+        $self->ws_client->connect( $endpoint )->cb( sub {
             $res->done( @_ )
         });
         $res
@@ -64,7 +68,7 @@ sub connect( $self, $handler, $got_endpoint, $logger ) {
         $logger->( 'trace', sprintf "Connected" );
         my $connection = $c->recv;
 
-        $self->{connection} = $connection;
+        $self->connection( $connection );
         undef $self;
 
         # Kick off the continous polling
